//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2008/03/14 12:04:00 $
//
// Category     : VST 2.x SDK Samples
// Filename     : stereoify.cpp
// Created by   : Robert Flack
// Description  : Audio effects and filters. Template base by Steinberg Media Technologies, adelay plugin.
//
//-------------------------------------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <cstdlib>

#ifndef __stereoify__
#include "stereoify.h"
#endif

//----------------------------------------------------------------------------- 
stereoifyProgram::stereoifyProgram ()
{
	// default Program Values
	fStereoify=0.0;
	fRemoveCenter=0.0;
	fRemoveStereo=0.0;
	fNoise=0.0;
	fSawtooth=0.0;
	strcpy (name, "Stereoify");
}

//-----------------------------------------------------------------------------
stereoify::stereoify (audioMasterCallback audioMaster)
	: AudioEffectX (audioMaster, kNumPrograms, kNumParams)
{
	// init	
	programs = new stereoifyProgram[numPrograms];
	fStereoify = fRemoveCenter = fRemoveStereo = fNoise = fSawtooth = 0.0;
	sawtooth_lastsample[0]=sawtooth_lastsample[1]=sawtooth_lastx[0]=sawtooth_lastx[1]=0;
	sawtooth_slope[0]=sawtooth_slope[0]=0.1;

	if (programs)
		setProgram (0);

	setNumInputs (2);	// mono input
	setNumOutputs (2);	// stereo output

	setUniqueID ('rf04');	// this should be unique, use the Steinberg web page for plugin Id registration

	resume ();		// flush buffer
}

//------------------------------------------------------------------------
stereoify::~stereoify ()
{
	if (programs)
		delete[] programs;
}

//------------------------------------------------------------------------
void stereoify::setProgram (VstInt32 program)
{
	stereoifyProgram* ap = &programs[program];

	curProgram = program;
	setParameter (kStereoify, ap->fStereoify);
	setParameter (kRemoveCenter, ap->fRemoveCenter);
	setParameter (kRemoveStereo, ap->fRemoveStereo);
	setParameter (kNoise, ap->fNoise);
	setParameter (kSawtooth, ap->fSawtooth);
}

//------------------------------------------------------------------------
void stereoify::setStereoify (float stereoifyamt)
{
	fStereoify = stereoifyamt;
	programs[curProgram].fStereoify = fStereoify;
}

void stereoify::setRemoveCenter(float RemoveCenteramt)
{
	fRemoveCenter = RemoveCenteramt;
	programs[curProgram].fRemoveCenter = fRemoveCenter;
}

void stereoify::setRemoveStereo(float RemoveStereoamt)
{
	fRemoveStereo = RemoveStereoamt;
	programs[curProgram].fRemoveStereo = fRemoveStereo;
}

void stereoify::setNoise(float Noiseamt)
{
	fNoise = Noiseamt;
	programs[curProgram].fNoise = fNoise;
}

void stereoify::setSawtooth(float Sawtoothamt)
{
	fSawtooth = Sawtoothamt;
	programs[curProgram].fSawtooth = Sawtoothamt;
}

//------------------------------------------------------------------------
void stereoify::setProgramName (char *name)
{
	strcpy (programs[curProgram].name, name);
}

//------------------------------------------------------------------------
void stereoify::getProgramName (char *name)
{
	if (!strcmp (programs[curProgram].name, "Init"))
		sprintf (name, "%s %d", programs[curProgram].name, curProgram + 1);
	else
		strcpy (name, programs[curProgram].name);
}

//-----------------------------------------------------------------------------------------
bool stereoify::getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text)
{
	if (index < kNumPrograms)
	{
		strcpy (text, programs[index].name);
		return true;
	}
	return false;
}

//------------------------------------------------------------------------
void stereoify::resume ()
{
	AudioEffectX::resume ();
}

//------------------------------------------------------------------------
void stereoify::setParameter (VstInt32 index, float value)
{
	stereoifyProgram* ap = &programs[curProgram];

	switch (index)
	{
		case kStereoify :    setStereoify (value);	break;
		case kRemoveCenter :	setRemoveCenter (value);	break;
		case kRemoveStereo :	setRemoveStereo (value);	break;
		case kNoise :	setNoise (value);	break;
		case kSawtooth : setSawtooth(value);	break;
	}
}

//------------------------------------------------------------------------
float stereoify::getParameter (VstInt32 index)
{
	float v = 0;

	switch (index)
	{
		case kStereoify :    v = fStereoify;	break;
		case kRemoveCenter :	v = fRemoveCenter;	break;
		case kRemoveStereo :	v = fRemoveStereo;	break;
		case kNoise :	v = fNoise;	break;
		case kSawtooth :	v = fSawtooth; break;
	}
	return v;
}

//------------------------------------------------------------------------
void stereoify::getParameterName (VstInt32 index, char *label)
{
	switch (index)
	{
		case kStereoify :    strcpy (label, "Stereoify");		break;
		case kRemoveCenter :	strcpy (label, "RemoveCenter");	break;
		case kRemoveStereo :	strcpy (label, "RemoveStereo");	break;
		case kNoise :	strcpy (label, "Noise");	break;
		case kSawtooth :	strcpy (label, "Sawtooth");	break;
	}
}

//------------------------------------------------------------------------
void stereoify::getParameterDisplay (VstInt32 index, char *text)
{
	switch (index)
	{
		case kStereoify : float2string (fStereoify, text, kVstMaxParamStrLen);	break;
		case kRemoveCenter : float2string (fRemoveCenter, text, kVstMaxParamStrLen);	break;
		case kRemoveStereo : float2string (fRemoveStereo, text, kVstMaxParamStrLen);	break;
		case kNoise : float2string (fNoise, text, kVstMaxParamStrLen);	break;
		case kSawtooth : float2string (fSawtooth, text, kVstMaxParamStrLen);	break;
	}
}

//------------------------------------------------------------------------
void stereoify::getParameterLabel (VstInt32 index, char *label)
{
	switch (index)
	{
		case kStereoify :    strcpy (label, "amount");	break;
		case kRemoveCenter :	strcpy (label, "percent"); break;
		case kRemoveStereo :	strcpy (label, "percent"); break;
		case kNoise :	strcpy (label, "amount");	break;
		case kSawtooth :	strcpy (label, "amount");	break;
	}
}

//------------------------------------------------------------------------
bool stereoify::getEffectName (char* name)
{
	strcpy (name, "Stereoify");
	return true;
}

//------------------------------------------------------------------------
bool stereoify::getProductString (char* text)
{
	strcpy (text, "Stereoify");
	return true;
}

//------------------------------------------------------------------------
bool stereoify::getVendorString (char* text)
{
	strcpy (text, "Robert Flack");
	return true;
}

//---------------------------------------------------------------------------
void stereoify::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
	float* in1 = inputs[0];
	float* in2 = inputs[1];
	float* out1 = outputs[0];
	float* out2 = outputs[1];
	float cslope[2];
	float x[2];

	// Process all sample frames
	while (--sampleFrames >= 0)
	{
		// Grab the next sample for each channel
		x[0] = *in1;
		x[1] = *in2;
		
		// Calculate a mono channel by averaging the left and right channel
		float mono = (float)((x[0] + x[1])*0.5);

		// Apply a filter to each channel
		for (int i=0;i<2;i++)
		{
			cslope[i]=x[i]-sawtooth_lastsample[i];

			// Check that the slope over the last sample is large enough
			if (sawtooth_slope[i]>0 && cslope[i]>0)
			{
				if (cslope[i]>sawtooth_slope[i])
					sawtooth_slope[i]=cslope[i];
				else
				{
					x[i]+=(sawtooth_slope[i]-(sawtooth_lastx[i]-x[i]))*fSawtooth;
				}
			}
			else if (sawtooth_slope[i]<0 && cslope[i]<0)
			{
				if (cslope[i]<sawtooth_slope[i])
					sawtooth_slope[i]=cslope[i];
				else
				{
					x[i]+=(sawtooth_slope[i]-(sawtooth_lastx[i]-x[i]))*fSawtooth;
				}
			}
			else
			{
				sawtooth_slope[i]*=-0.75;
				//sawtooth_slope[i]=cslope[i];
			}
			sawtooth_lastx[i]=x[i];
			
			// Decay the required slope
			sawtooth_slope[i]*=0.95;
			// sawtooth_slope[i]+=0.15*cslope[i];
		}
		
		sawtooth_lastsample[0]=*in1;
		sawtooth_lastsample[1]=*in2;

		// Subtract out mono signal from each left and right signal by weight
		// and add in extra stereo sound
		*out1 = x[0] - mono*fRemoveCenter + (x[0] - mono)*fStereoify*4 - (x[0] - mono)*fRemoveStereo;
		*out2 = x[1] - mono*fRemoveCenter + (x[1] - mono)*fStereoify*4 - (x[1] - mono)*fRemoveStereo;

		// Add noise according to the signal amplitude
		*out1*=(float)(1.0+((float)rand()/RAND_MAX-0.5)*fNoise);
		*out2*=(float)(1.0+((float)rand()/RAND_MAX-0.5)*fNoise);

		// Step to next input and output sample
		out1++;
		out2++;
		in1++;
		in2++;
	}
}
