//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Version 2.4		$Date: 2008/03/14 12:04:00 $
//
// Category     : VST 2.x SDK Samples
// Filename     : stereoify.h
// Created by   : Robert Flack
// Description  : Audio effects and filters. Template base by Steinberg Media Technologies, adelay plugin.
//
//-------------------------------------------------------------------------------------------------------

#ifndef __stereoify__
#define __stereoify__

#include "public.sdk/source/vst2.x/audioeffectx.h"

// Enumeration type for the various effects of this filter
enum
{
	// Global
	kNumPrograms = 16,

	// Parameters Tags
	kStereoify = 0,
	kRemoveCenter,
	kRemoveStereo,
	kNoise,
	kSawtooth,

	kNumParams
};

class stereoify;

//------------------------------------------------------------------------
class stereoifyProgram
{
friend class stereoify;
public:
	stereoifyProgram ();
	~stereoifyProgram () {}

private:	
	// Store the amounts of each control
	float fStereoify;
	float fRemoveCenter;
	float fRemoveStereo;
	float fNoise;
	float fSawtooth;
	char name[24];
};

//------------------------------------------------------------------------
class stereoify : public AudioEffectX
{
public:
	stereoify (audioMasterCallback audioMaster);
	~stereoify ();

	//---from AudioEffect-----------------------
	// Processes the audio clip passed into it with the filter
	virtual void processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames);

	// Standard VST methods to support integration with an audio program

	virtual void setProgram (VstInt32 program);
	virtual void setProgramName (char* name);
	virtual void getProgramName (char* name);
	virtual bool getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text);
	
	virtual void setParameter (VstInt32 index, float value);
	virtual float getParameter (VstInt32 index);
	virtual void getParameterLabel (VstInt32 index, char* label);
	virtual void getParameterDisplay (VstInt32 index, char* text);
	virtual void getParameterName (VstInt32 index, char* text);

	virtual void resume ();

	virtual bool getEffectName (char* name);
	virtual bool getVendorString (char* text);
	virtual bool getProductString (char* text);
	virtual VstInt32 getVendorVersion () { return 1000; }
	
	virtual VstPlugCategory getPlugCategory () { return kPlugCategEffect; }

protected:
	void setStereoify (float stereoifyamt);
	void setRemoveCenter (float RemoveCenteramt);
	void setRemoveStereo (float RemoveStereoamt);
	void setNoise (float Noiseamt);
	void setSawtooth (float Sawtoothamt);

	stereoifyProgram* programs;

	float fStereoify;	
	float fRemoveCenter;
	float fRemoveStereo;
	float fNoise;
	float fSawtooth;

	float sawtooth_lastsample[2];
	float sawtooth_slope[2];
	float sawtooth_lastx[2];
};

#endif
